@@ -21,6 +21,8 @@ package com.eteks.sweethome3d;
 
 import java.awt.EventQueue;
 import java.awt.KeyboardFocusManager;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.BufferedReader;
@@ -36,8 +38,10 @@ import java.net.URI;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.concurrent.Executor;
@@ -55,6 +59,7 @@ import javax.swing.JComponent;
 import javax.swing.JFrame;
 import javax.swing.JOptionPane;
 import javax.swing.SwingUtilities;
+import javax.swing.Timer;
 import javax.swing.UIManager;
 
 import com.eteks.sweethome3d.io.AutoRecoveryManager;
@@ -66,6 +71,7 @@ import com.eteks.sweethome3d.model.CollectionListener;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeApplication;
 import com.eteks.sweethome3d.model.HomeRecorder;
+import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.RecorderException;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.plugin.HomePluginController;
@@ -138,6 +144,7 @@ public class SweetHome3D extends HomeApplication {
   private ViewFactory             viewFactory;
   private PluginManager           pluginManager;
   private boolean                 pluginManagerInitialized;
+  private boolean                 checkUpdatesNeeded;
   private AutoRecoveryManager     autoRecoveryManager;
   private final Map<Home, JFrame> homeFrames;
 
@@ -198,12 +205,39 @@ public class SweetHome3D extends HomeApplication {
       } else {
         applicationFolders = null;
       }
-      this.userPreferences = new FileUserPreferences(preferencesFolder, applicationFolders, 
-          new Executor() {
-            public void execute(Runnable command) {
-              EventQueue.invokeLater(command);
+      Executor eventQueueExecutor = new Executor() {
+          public void execute(Runnable command) {
+            EventQueue.invokeLater(command);
+          }
+        };
+      this.userPreferences = new FileUserPreferences(preferencesFolder, applicationFolders, eventQueueExecutor) {
+          @Override
+          public List<Library> getLibraries() {
+            if (pluginManager != null) {
+              List<Library> pluginLibraries = pluginManager.getPluginLibraries();
+              if (!pluginLibraries.isEmpty()) {
+                // Add plug-ins to the list returned by user preferences
+                ArrayList<Library> libraries = new ArrayList<Library>(super.getLibraries());
+                libraries.addAll(pluginLibraries);
+                return Collections.unmodifiableList(libraries);
+              }
             }
-          });
+            return super.getLibraries();
+          }
+          
+          @Override
+          public void deleteLibraries(List<Library> libraries) throws RecorderException {
+            super.deleteLibraries(libraries);
+            List<Library> plugins = new ArrayList<Library>();
+            for (Library library : libraries) {
+              if (PluginManager.PLUGIN_LIBRARY_TYPE.equals(library.getType())) {
+                plugins.add(library);
+              }
+            }
+            pluginManager.deletePlugins(plugins);            
+          }
+        };
+      this.checkUpdatesNeeded = this.userPreferences.isCheckUpdatesEnabled();
     }
     return this.userPreferences;
   }
@@ -248,6 +282,23 @@ public class SweetHome3D extends HomeApplication {
     return this.pluginManager;
   }
 
+  /**
+   * Returns Sweet Home 3D application read from resources.
+   */
+  @Override
+  public String getId() {
+    String applicationId = System.getProperty("com.eteks.sweethome3d.applicationId");
+    if (applicationId != null && applicationId.length() > 0) {
+      return applicationId;
+    } else {
+      try {
+        return getUserPreferences().getLocalizedString(SweetHome3D.class, "applicationId");
+      } catch (IllegalArgumentException ex) {
+        return super.getId();
+      }
+    }
+  }
+  
   /**
    * Returns the name of this application read from resources.
    */
@@ -260,12 +311,12 @@ public class SweetHome3D extends HomeApplication {
    * Returns information about the version of this application.
    */
   public String getVersion() {
-    String applicationVersion = getUserPreferences().getLocalizedString(SweetHome3D.class, "applicationVersion");
-    String versionInformation = System.getProperty("com.eteks.sweethome3d.deploymentInformation");
-    if (versionInformation != null) {
-      applicationVersion += " " + versionInformation;
+    String applicationVersion = System.getProperty("com.eteks.sweethome3d.applicationVersion");
+    if (applicationVersion != null) {
+      return applicationVersion;
+    } else {
+      return getUserPreferences().getLocalizedString(SweetHome3D.class, "applicationVersion");
     }
-    return applicationVersion;
   }
 
   /**
@@ -339,7 +390,7 @@ public class SweetHome3D extends HomeApplication {
 
     // Add a listener that opens a frame when a home is added to application
     addHomesListener(new CollectionListener<Home>() {
-      private boolean firstApplicationHomeAdded;
+      private boolean        firstApplicationHomeAdded;
 
       public void collectionChanged(CollectionEvent<Home> ev) {
         switch (ev.getType()) {
@@ -349,11 +400,11 @@ public class SweetHome3D extends HomeApplication {
               HomeFrameController controller = createHomeFrameController(home);
               controller.displayView();
               if (!this.firstApplicationHomeAdded) {
-                addNewHomeCloseListener(home, controller.getHomeController());
                 this.firstApplicationHomeAdded = true;
+                addNewHomeCloseListener(home, controller.getHomeController());
               }
 
-              JFrame homeFrame = (JFrame) SwingUtilities.getRoot((JComponent) controller.getView());
+              JFrame homeFrame = (JFrame)SwingUtilities.getRoot((JComponent) controller.getView());
               homeFrames.put(home, homeFrame);
             } catch (IllegalStateException ex) {
               // Check exception by class name to avoid a mandatory bind to Java 3D
@@ -431,12 +482,16 @@ public class SweetHome3D extends HomeApplication {
       ResourceBundle resource = ResourceBundle.getBundle(classPackage + "." + "package");
       String applicationName = resource.getString("SweetHome3D.applicationName");
       System.setProperty("com.apple.mrj.application.apple.menu.about.name", applicationName);
+      System.setProperty("apple.awt.application.name", applicationName);
       // Use Mac OS X screen menu bar for frames menu bar
       System.setProperty("apple.laf.useScreenMenuBar", "true");
       // Force the use of Quartz under Mac OS X for better Java 2D rendering
       // performance
       System.setProperty("apple.awt.graphics.UseQuartz", "true");
     }
+    // Set User Agent to follow statistics on used operating systems 
+    System.setProperty("http.agent", getId() + "/" + getVersion()  
+         + " (" + System.getProperty("os.name") + " " + System.getProperty("os.version") + "; " + System.getProperty("os.arch") + "; " + Locale.getDefault() + ")");
   }
 
   /**
@@ -613,8 +668,9 @@ public class SweetHome3D extends HomeApplication {
           });
         // Read home file in args [1] if args [0] == "-open" with a dummy controller
         createHomeFrameController(createHome()).getHomeController().open(args [1]);
+        checkUpdates();
       } else if (getContentManager().isAcceptable(args [1], ContentManager.ContentType.LANGUAGE_LIBRARY)) {
-        start(new String [0]);
+        showDefaultHomeFrame();
         final String languageLibraryName = args [1];
         EventQueue.invokeLater(new Runnable() {
           public void run() {
@@ -628,28 +684,31 @@ public class SweetHome3D extends HomeApplication {
                 break;
               }
             }
+            checkUpdates();
           }
         });
       } else if (getContentManager().isAcceptable(args [1], ContentManager.ContentType.FURNITURE_LIBRARY)) {
-        start(new String [0]);
+        showDefaultHomeFrame();
         final String furnitureLibraryName = args [1];
         EventQueue.invokeLater(new Runnable() {
           public void run() {
             // Import furniture library with a dummy controller
             createHomeFrameController(createHome()).getHomeController().importFurnitureLibrary(furnitureLibraryName);
+            checkUpdates();
           }
         });
       } else if (getContentManager().isAcceptable(args [1], ContentManager.ContentType.TEXTURES_LIBRARY)) {
-        start(new String [0]);
+        showDefaultHomeFrame();
         final String texturesLibraryName = args [1];
         EventQueue.invokeLater(new Runnable() {
           public void run() {
             // Import textures library with a dummy controller
             createHomeFrameController(createHome()).getHomeController().importTexturesLibrary(texturesLibraryName);
+            checkUpdates();
           }
         });
       } else if (getContentManager().isAcceptable(args [1], ContentManager.ContentType.PLUGIN)) {
-        start(new String [0]);
+        showDefaultHomeFrame();
         final String pluginName = args [1];
         EventQueue.invokeLater(new Runnable() {
           public void run() {
@@ -658,11 +717,22 @@ public class SweetHome3D extends HomeApplication {
             if (homeController instanceof HomePluginController) {
               ((HomePluginController)homeController).importPlugin(pluginName);
             }
+            checkUpdates();
           }
         });
       }
-    } else if (getHomes().isEmpty()) {
-      if (autoRecoveryManager != null) {
+    } else { 
+      showDefaultHomeFrame();
+      checkUpdates();
+    }
+  }
+
+  /**
+   * Shows a home frame, either a new one when no home is opened, or the last created home frame.  
+   */
+  public void showDefaultHomeFrame() {
+    if (getHomes().isEmpty()) {
+      if (this.autoRecoveryManager != null) {
         this.autoRecoveryManager.openRecoveredHomes();
       }
       if (getHomes().isEmpty()) {
@@ -690,11 +760,28 @@ public class SweetHome3D extends HomeApplication {
           }
         }
       }
-
+ 
       showHomeFrame(home);
     }
   }
 
+  /**
+   * Check updates if needed.
+   */
+  private void checkUpdates() {
+    if (this.checkUpdatesNeeded) {
+      this.checkUpdatesNeeded = false;
+      // Delay updates checking to let program launch finish
+      new Timer(500, new ActionListener() {
+          public void actionPerformed(ActionEvent ev) {
+            ((Timer)ev.getSource()).stop();
+            // Check updates with a dummy controller
+            createHomeFrameController(createHome()).getHomeController().checkUpdates(true);
+          }
+        }).start();
+    }
+  }
+  
   /**
    * A file content manager that records the last directories for each content
    * in Java preferences.
@@ -791,24 +878,27 @@ public class SweetHome3D extends HomeApplication {
    */
   private static class StandaloneBasicService implements BasicService {
     public boolean showDocument(URL url) {
-      if (isJava6()) {
-        try {
+      try {
+        if (OperatingSystem.isJavaVersionGreaterOrEqual("1.6")) {
           // Call Java SE 6 java.awt.Desktop browse method by reflection to
           // ensure Java SE 5 compatibility
-          Class desktopClass = Class.forName("java.awt.Desktop");
+          Class<?> desktopClass = Class.forName("java.awt.Desktop");
           Object desktopInstance = desktopClass.getMethod("getDesktop").invoke(null);
           desktopClass.getMethod("browse", URI.class).invoke(desktopInstance, url.toURI());
           return true;
-        } catch (Exception ex) {
-          // For any exception, let's consider simply the showDocument method
-          // failed
         }
-      } else if (OperatingSystem.isMacOSX()) {
+      } catch (Exception ex) {
         try {
-          Runtime.getRuntime().exec(new String [] {"open", url.toString()});
-          return true;
-        } catch (IOException ex) {
+          if (OperatingSystem.isMacOSX()) {
+            Runtime.getRuntime().exec(new String [] {"open", url.toString()});
+            return true;
+          } else if (OperatingSystem.isLinux()) {
+            Runtime.getRuntime().exec(new String [] {"xdg-open", url.toString()});
+            return true;
+          }  
+        } catch (IOException ex2) {
         }
+        // For other cases, let's consider simply the showDocument method failed
       }
       return false;
     }
@@ -823,38 +913,24 @@ public class SweetHome3D extends HomeApplication {
     }
 
     public boolean isWebBrowserSupported() {
-      if (isJava6()) {
+      if (OperatingSystem.isJavaVersionGreaterOrEqual("1.6")) {
         try {
           // Call Java SE 6 java.awt.Desktop isSupported(Desktop.Action.BROWSE)
-          // method by reflection to
-          // ensure Java SE 5 compatibility
-          Class desktopClass = Class.forName("java.awt.Desktop");
+          // method by reflection to ensure Java SE 5 compatibility
+          Class<?> desktopClass = Class.forName("java.awt.Desktop");
           Object desktopInstance = desktopClass.getMethod("getDesktop").invoke(null);
-          Class desktopActionClass = Class.forName("java.awt.Desktop$Action");
+          Class<?> desktopActionClass = Class.forName("java.awt.Desktop$Action");
           Object desktopBrowseAction = desktopActionClass.getMethod("valueOf", String.class).invoke(null, "BROWSE");
-          return (Boolean) desktopClass.getMethod("isSupported", desktopActionClass).invoke(desktopInstance,
-              desktopBrowseAction);
+          if ((Boolean)desktopClass.getMethod("isSupported", desktopActionClass).invoke(desktopInstance,
+              desktopBrowseAction)) {
+            return true;
+          }
         } catch (Exception ex) {
           // For any exception, let's consider simply the isSupported method failed
         }
       }
-      // For other Java versions, let's support only Mac OS X
-      return OperatingSystem.isMacOSX();
-    }
-
-    private boolean isJava6() {
-      String javaVersion = System.getProperty("java.version");
-      String [] javaVersionParts = javaVersion.split("\\.|_");
-      if (javaVersionParts.length >= 2) {
-        try {
-          // Return true for Java SE 6 and superior
-          if (Integer.parseInt(javaVersionParts [1]) >= 6) {
-            return true;
-          }
-        } catch (NumberFormatException ex) {
-        }
-      }
-      return false;
+      // For other Java versions, let's support Mac OS X and Linux
+      return OperatingSystem.isMacOSX() || OperatingSystem.isLinux();
     }
   }
 
