@@ -46,9 +46,11 @@ import com.eteks.sweethome3d.model.CatalogPieceOfFurniture;
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.FurnitureCategory;
+import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.LightSource;
 import com.eteks.sweethome3d.model.Sash;
 import com.eteks.sweethome3d.model.UserPreferences;
+import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
@@ -88,7 +90,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      */
     TAGS("tags"),
     /**
-     * The key for the creation date of a piece of furniture (optional).
+     * The key for the creation date of a piece of furniture at <code>yyyy-MM-dd</code> format (optional).
      */
     CREATION_DATE("creationDate"),
     /**
@@ -124,7 +126,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     /**
      * The key for a piece of furniture with multiple parts (optional).
      * If the value of this key is <code>true</code>, all the files
-     * stored in the same directory as the 3D model file (MTL, texture files...)
+     * stored in the same folder as the 3D model file (MTL, texture files...)
      * will be considered as being necessary to view correctly the 3D model. 
      */
     MULTI_PART_MODEL("multiPartModel"),
@@ -287,6 +289,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
   private static final String CONTRIBUTED_FURNITURE_CATALOG_FAMILY = "ContributedFurnitureCatalog";
   private static final String ADDITIONAL_FURNITURE_CATALOG_FAMILY  = "AdditionalFurnitureCatalog";
   
+  private List<Library> libraries = new ArrayList<Library>();
+  
   /**
    * Creates a default furniture catalog read from resources in the package of this class.
    */
@@ -333,9 +337,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         });
         
         if (pluginFurnitureCatalogFiles != null) {
-          // Treat furniture catalog files in reverse order so file named with a date will be taken into account 
-          // from most recent to least recent
-          Arrays.sort(pluginFurnitureCatalogFiles, Collections.reverseOrder());
+          // Treat furniture catalog files in reverse order of their version
+          Arrays.sort(pluginFurnitureCatalogFiles, Collections.reverseOrder(OperatingSystem.getFileVersionComparator()));
           for (File pluginFurnitureCatalogFile : pluginFurnitureCatalogFiles) {
             // Try to load the properties file describing furniture catalog from current file  
             readPluginFurnitureCatalog(pluginFurnitureCatalogFile, identifiedFurniture);
@@ -370,6 +373,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         try {        
           ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
               new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+          this.libraries.add(0, new DefaultLibrary(pluginFurnitureCatalogUrl.toExternalForm(), 
+              UserPreferences.FURNITURE_LIBRARY_TYPE, resource));
           readFurniture(resource, pluginFurnitureCatalogUrl, furnitureResourcesUrlBase, identifiedFurniture);
         } catch (MissingResourceException ex) {
           // Ignore malformed furniture catalog
@@ -383,6 +388,14 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       readFurniture(resource, null, furnitureResourcesUrlBase, identifiedFurniture);
     }
   }
+  
+  /**
+   * Returns the furniture libraries at initialization.
+   * @since 4.0 
+   */
+  public List<Library> getLibraries() {
+    return Collections.unmodifiableList(this.libraries);
+  }
 
   private static final Map<File,URL> pluginFurnitureCatalogUrlUpdates = new HashMap<File,URL>(); 
   
@@ -409,7 +422,9 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       }
       
       ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
-          new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));      
+          new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+      this.libraries.add(0, new DefaultLibrary(pluginFurnitureCatalogFile.getCanonicalPath(), 
+          UserPreferences.FURNITURE_LIBRARY_TYPE, resourceBundle));
       readFurniture(resourceBundle, pluginFurnitureCatalogUrl, null, identifiedFurniture);
     } catch (MissingResourceException ex) {
       // Ignore malformed furniture catalog
@@ -630,7 +645,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
    * @param resourceUrlBase the URL used as a base to build the URL to content file  
    *            or <code>null</code> if it's read from current classpath or <code>furnitureCatalogUrl</code>.
    * @param multiPartModel if <code>true</code> the resource is a multi part resource stored 
-   *                 in a directory with other required resources
+   *                 in a folder with other required resources
    * @throws IllegalArgumentException if the file value doesn't match a valid resource or URL.
    */
   private Content getContent(ResourceBundle resource, 
