@@ -39,9 +39,11 @@ import java.util.ResourceBundle;
 
 import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.TexturesCatalog;
 import com.eteks.sweethome3d.model.TexturesCategory;
 import com.eteks.sweethome3d.model.UserPreferences;
+import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
@@ -112,6 +114,8 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
 
   private static final String ADDITIONAL_TEXTURES_CATALOG_FAMILY  = "AdditionalTexturesCatalog";
 
+  private List<Library> libraries = new ArrayList<Library>();
+  
   /**
    * Creates a default textures catalog read from resources.
    */
@@ -156,9 +160,8 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
         });
         
         if (pluginTexturesCatalogFiles != null) {
-          // Treat textures catalog files in reverse order so file named with a date will be taken into account 
-          // from most recent to least recent
-          Arrays.sort(pluginTexturesCatalogFiles, Collections.reverseOrder());
+          // Treat textures catalog files in reverse order of their version
+          Arrays.sort(pluginTexturesCatalogFiles, Collections.reverseOrder(OperatingSystem.getFileVersionComparator()));
           for (File pluginTexturesCatalogFile : pluginTexturesCatalogFiles) {
             // Try to load the properties file describing textures catalog from current file  
             readPluginTexturesCatalog(pluginTexturesCatalogFile, identifiedTextures);
@@ -192,6 +195,8 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
         try {
           ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
                   new URLClassLoader(new URL [] {pluginTexturesCatalogUrl}));
+          this.libraries.add(0, new DefaultLibrary(pluginTexturesCatalogUrl.toExternalForm(), 
+              UserPreferences.TEXTURES_LIBRARY_TYPE, resource));
           readTextures(resource, pluginTexturesCatalogUrl, texturesResourcesUrlBase, identifiedTextures);
         } catch (MissingResourceException ex) {
           // Ignore malformed textures catalog
@@ -206,6 +211,14 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
     }
   }
 
+  /**
+   * Returns the furniture libraries at initialization.
+   * @since 4.0 
+   */
+  public List<Library> getLibraries() {
+    return Collections.unmodifiableList(this.libraries);
+  }
+
   private static final Map<File,URL> pluginTexturesCatalogUrlUpdates = new HashMap<File,URL>(); 
   
   /**
@@ -233,6 +246,8 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
       
       ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
           new URLClassLoader(new URL [] {pluginTexturesCatalogUrl}));      
+      this.libraries.add(0, new DefaultLibrary(pluginTexturesCatalogFile.getCanonicalPath(), 
+          UserPreferences.TEXTURES_LIBRARY_TYPE, resourceBundle));
       readTextures(resourceBundle, pluginTexturesCatalogUrl, null, identifiedTextures);
     } catch (MissingResourceException ex) {
       // Ignore malformed textures catalog
