@@ -35,6 +35,7 @@ import java.io.Writer;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.text.NumberFormat;
+import java.util.Collection;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
@@ -85,7 +86,7 @@ import javax.vecmath.Vector4f;
 /**
  * An output stream that writes Java 3D nodes at OBJ + MTL format.
  * <p>Once you wrote nodes, call <code>close</code> method to create the MTL file
- * and the texture images in the same directory as OBJ file. This feature applies
+ * and the texture images in the same folder as OBJ file. This feature applies
  * only to constructor that takes a file as parameter.<br>
  * Note: this class is compatible with Java 3D 1.3.
  * @author Emmanuel Puybaret
@@ -370,7 +371,25 @@ public class OBJWriter extends FilterWriter {
             String appearanceName = this.appearances.get(comparableAppearance);
             if (appearanceName == null) {
               // Store appearance
-              appearanceName = objectName;
+              try {
+                appearanceName = appearance.getName();
+              } catch (NoSuchMethodError ex) {
+                // Don't reuse appearance name with Java 3D < 1.4 where getName was added              }
+              }
+              if (appearanceName == null || !accept(appearanceName)) {
+                appearanceName = objectName;
+              } else {
+                // Find a unique appearance name among appearances 
+                Collection<String> appearanceNames = this.appearances.values();
+                String baseName = appearanceName + "_" + objectName;
+                for (int i = 0; appearanceNames.contains(appearanceName); i++) {
+                  if (i == 0) {
+                    appearanceName = baseName;
+                  } else {
+                    appearanceName = baseName + "_" + i;
+                  }
+                }
+              }
               this.appearances.put(comparableAppearance, appearanceName);
               
               Texture texture = appearance.getTexture();
@@ -1415,6 +1434,20 @@ public class OBJWriter extends FilterWriter {
             return false;
           }
         }
+        // Compare name
+        try {
+          String name1 = this.appearance.getName();
+          String name2 = appearance2.getName();
+          if ((name1 == null) ^ (name2 == null)) {
+            return false;
+          } else if (name1 != name2 
+                     && !name1.equals(name2)) {
+            return false;
+          }
+        } catch (NoSuchMethodError ex) {
+          // Don't compares name with Java 3D < 1.4 where getName was added              }
+        }
+
         return true;
       }
       return false;
@@ -1450,6 +1483,14 @@ public class OBJWriter extends FilterWriter {
       if (texture != null) {
         code += texture.getImage(0).hashCode();
       }
+      try {
+        String name = this.appearance.getName();
+        if (name != null) {
+          code += name.hashCode();
+        }
+      } catch (NoSuchMethodError ex) {
+        // Don't take name into account with Java 3D < 1.4 where getName was added              }
+      }
       return code;
     }
   }
