@@ -46,7 +46,7 @@ public class Home implements Serializable, Cloneable {
    * in <code>Home</code> class or in one of the classes that it uses,
    * this number is increased.
    */
-  public static final long CURRENT_VERSION = 3700;
+  public static final long CURRENT_VERSION = 4000;
   
   private static final boolean KEEP_BACKWARD_COMPATIBLITY = true;
 
@@ -254,7 +254,7 @@ public class Home implements Serializable, Cloneable {
           HomePieceOfFurniture.SortableProperty.VISIBLE});
     }
     // Create a default top camera that matches default point of view in previous versions 
-    this.topCamera = new Camera(500, 1500, 1000, 
+    this.topCamera = new Camera(500, 1500, 1010, 
         (float)Math.PI, (float)Math.PI / 4, (float)Math.PI * 63 / 180);
     // Create a default observer camera (use a 63 field of view equivalent to a 35mm lens for a 24x36 film)
     this.observerCamera = new ObserverCamera(50, 50, 170, 
@@ -313,79 +313,82 @@ public class Home implements Serializable, Cloneable {
     this.version = CURRENT_VERSION;
         
     if (KEEP_BACKWARD_COMPATIBLITY) {
-      HomePieceOfFurniture.SortableProperty currentFurnitureSortedProperty = this.furnitureSortedProperty;
-      if (this.furnitureSortedProperty != null) {
-        this.furnitureSortedPropertyName = this.furnitureSortedProperty.name();
-        // Store in furnitureSortedProperty only backward compatible property
-        if (!isFurnitureSortedPropertyBackwardCompatible(this.furnitureSortedProperty)) {
-          this.furnitureSortedProperty = null;
+      HomePieceOfFurniture.SortableProperty homeFurnitureSortedProperty = this.furnitureSortedProperty;
+      List<HomePieceOfFurniture.SortableProperty> homeFurnitureVisibleProperties = this.furnitureVisibleProperties;
+      List<HomePieceOfFurniture> homeFurniture = this.furniture;
+      try {
+        if (this.furnitureSortedProperty != null) {
+          this.furnitureSortedPropertyName = this.furnitureSortedProperty.name();
+          // Store in furnitureSortedProperty only backward compatible property
+          if (!isFurnitureSortedPropertyBackwardCompatible(this.furnitureSortedProperty)) {
+            this.furnitureSortedProperty = null;
+          }
         }
-      }
-      
-      this.furnitureVisiblePropertyNames = new ArrayList<String>();
-      // Store in furnitureVisibleProperties only backward compatible properties
-      List<HomePieceOfFurniture.SortableProperty> currentFurnitureVisibleProperties = this.furnitureVisibleProperties;
-      this.furnitureVisibleProperties = new ArrayList<HomePieceOfFurniture.SortableProperty>();
-      for (HomePieceOfFurniture.SortableProperty visibleProperty : currentFurnitureVisibleProperties) {
-        this.furnitureVisiblePropertyNames.add(visibleProperty.name());
-        if (isFurnitureSortedPropertyBackwardCompatible(visibleProperty)) {
-          this.furnitureVisibleProperties.add(visibleProperty);
-        }
-      }
-    
-      // Store referenced HomeFurnitureGroup instances in a separate field 
-      // for backward compatibility reasons (version < 2.3)
-      this.furnitureWithGroups = this.furniture;
-      // Serialize a furnitureWithDoorsAndWindows field that contains only 
-      // HomePieceOfFurniture, HomeDoorOrWindow and HomeLight instances
-      // for backward compatibility reasons (version < 1.7)
-      this.furnitureWithDoorsAndWindows = new ArrayList<HomePieceOfFurniture>(this.furniture.size());
-      // Serialize a furniture field that contains only HomePieceOfFurniture instances
-      this.furniture = new ArrayList<HomePieceOfFurniture>(this.furniture.size());
-      for (HomePieceOfFurniture piece : this.furnitureWithGroups) {
-        if (piece.getClass() == HomePieceOfFurniture.class) {
-          this.furnitureWithDoorsAndWindows.add(piece);
-          this.furniture.add(piece);
-        } else {
-          if (piece.getClass() == HomeFurnitureGroup.class) {
-            // Add the ungrouped pieces to furniture and furnitureWithDoorsAndWindows list 
-            for (HomePieceOfFurniture groupPiece : getGroupFurniture((HomeFurnitureGroup)piece)) {
-              this.furnitureWithDoorsAndWindows.add(groupPiece);
-              if (groupPiece.getClass() == HomePieceOfFurniture.class) {
-                this.furniture.add(groupPiece);
-              } else {
-                // Create backward compatible instances
-                this.furniture.add(new HomePieceOfFurniture(groupPiece));
-              }
-            }            
-          } else {
-            this.furnitureWithDoorsAndWindows.add(piece);
-            // Create backward compatible instances
-            this.furniture.add(new HomePieceOfFurniture(piece));
+        
+        this.furnitureVisiblePropertyNames = new ArrayList<String>();
+        // Store in furnitureVisibleProperties only backward compatible properties
+        this.furnitureVisibleProperties = new ArrayList<HomePieceOfFurniture.SortableProperty>();
+        for (HomePieceOfFurniture.SortableProperty visibleProperty : homeFurnitureVisibleProperties) {
+          this.furnitureVisiblePropertyNames.add(visibleProperty.name());
+          if (isFurnitureSortedPropertyBackwardCompatible(visibleProperty)) {
+            this.furnitureVisibleProperties.add(visibleProperty);
           }
         }
-      }
       
-      // Store environment fields in home fields for compatibility reasons
-      this.groundColor = this.environment.getGroundColor();
-      this.groundTexture = this.environment.getGroundTexture();
-      this.skyColor = this.environment.getSkyColor();
-      this.lightColor = this.environment.getLightColor();
-      this.wallsAlpha = this.environment.getWallsAlpha();
-
-      out.defaultWriteObject();
-    
-      // Restore current values
-      this.furniture = this.furnitureWithGroups;
-      this.furnitureWithDoorsAndWindows = null;
-      this.furnitureWithGroups = null;
-    
-      this.furnitureSortedProperty = currentFurnitureSortedProperty;
-      this.furnitureVisibleProperties = currentFurnitureVisibleProperties;
-      // Set furnitureSortedPropertyName and furnitureVisiblePropertyNames to null
-      // (they are used only for serialization)
-      this.furnitureSortedPropertyName = null;
-      this.furnitureVisiblePropertyNames = null;
+        // Store referenced HomeFurnitureGroup instances in a separate field 
+        // for backward compatibility reasons (version < 2.3)
+        this.furnitureWithGroups = this.furniture;
+        // Serialize a furnitureWithDoorsAndWindows field that contains only 
+        // HomePieceOfFurniture, HomeDoorOrWindow and HomeLight instances
+        // for backward compatibility reasons (version < 1.7)
+        this.furnitureWithDoorsAndWindows = new ArrayList<HomePieceOfFurniture>(this.furniture.size());
+        // Serialize a furniture field that contains only HomePieceOfFurniture instances
+        this.furniture = new ArrayList<HomePieceOfFurniture>(this.furniture.size());
+        for (HomePieceOfFurniture piece : this.furnitureWithGroups) {
+          if (piece.getClass() == HomePieceOfFurniture.class) {
+            this.furnitureWithDoorsAndWindows.add(piece);
+            this.furniture.add(piece);
+          } else {
+            if (piece.getClass() == HomeFurnitureGroup.class) {
+              // Add the ungrouped pieces to furniture and furnitureWithDoorsAndWindows list 
+              for (HomePieceOfFurniture groupPiece : getGroupFurniture((HomeFurnitureGroup)piece)) {
+                this.furnitureWithDoorsAndWindows.add(groupPiece);
+                if (groupPiece.getClass() == HomePieceOfFurniture.class) {
+                  this.furniture.add(groupPiece);
+                } else {
+                  // Create backward compatible instances
+                  this.furniture.add(new HomePieceOfFurniture(groupPiece));
+                }
+              }            
+            } else {
+              this.furnitureWithDoorsAndWindows.add(piece);
+              // Create backward compatible instances
+              this.furniture.add(new HomePieceOfFurniture(piece));
+            }
+          }
+        }
+        
+        // Store environment fields in home fields for compatibility reasons
+        this.groundColor = this.environment.getGroundColor();
+        this.groundTexture = this.environment.getGroundTexture();
+        this.skyColor = this.environment.getSkyColor();
+        this.lightColor = this.environment.getLightColor();
+        this.wallsAlpha = this.environment.getWallsAlpha();
+  
+        out.defaultWriteObject();
+      } finally {
+        // Restore home values
+        this.furniture = homeFurniture;
+        this.furnitureWithDoorsAndWindows = null;
+        this.furnitureWithGroups = null;
+      
+        this.furnitureSortedProperty = homeFurnitureSortedProperty;
+        this.furnitureVisibleProperties = homeFurnitureVisibleProperties;
+        // Set furnitureSortedPropertyName and furnitureVisiblePropertyNames to null
+        // (they are used only for serialization)
+        this.furnitureSortedPropertyName = null;
+        this.furnitureVisiblePropertyNames = null;
+      }
     } else {
       out.defaultWriteObject();
     }
@@ -787,11 +790,14 @@ public class Home implements Serializable, Cloneable {
         otherWall.setWallAtEnd(null);
       }
     }
-    wall.setLevel(null);
-    // Make a copy of the list to avoid conflicts in the list returned by getWalls
-    this.walls = new ArrayList<Wall>(this.walls);
-    this.walls.remove(wall);
-    this.wallsChangeSupport.fireCollectionChanged(wall, CollectionEvent.Type.DELETE);
+    int index = this.walls.indexOf(wall);
+    if (index != -1) {
+      wall.setLevel(null);
+      // Make a copy of the list to avoid conflicts in the list returned by getWalls
+      this.walls = new ArrayList<Wall>(this.walls);
+      this.walls.remove(index);
+      this.wallsChangeSupport.fireCollectionChanged(wall, CollectionEvent.Type.DELETE);
+    }
   }
 
   /**
@@ -842,11 +848,14 @@ public class Home implements Serializable, Cloneable {
   public void deleteDimensionLine(DimensionLine dimensionLine) {
     //  Ensure selectedItems don't keep a reference to dimension line
     deselectItem(dimensionLine);
-    dimensionLine.setLevel(null);
-    // Make a copy of the list to avoid conflicts in the list returned by getDimensionLines
-    this.dimensionLines = new ArrayList<DimensionLine>(this.dimensionLines);
-    this.dimensionLines.remove(dimensionLine);
-    this.dimensionLinesChangeSupport.fireCollectionChanged(dimensionLine, CollectionEvent.Type.DELETE);
+    int index = this.dimensionLines.indexOf(dimensionLine);
+    if (index != -1) {
+      dimensionLine.setLevel(null);
+      // Make a copy of the list to avoid conflicts in the list returned by getDimensionLines
+      this.dimensionLines = new ArrayList<DimensionLine>(this.dimensionLines);
+      this.dimensionLines.remove(index);
+      this.dimensionLinesChangeSupport.fireCollectionChanged(dimensionLine, CollectionEvent.Type.DELETE);
+    }
   }
 
   /**
@@ -896,11 +905,14 @@ public class Home implements Serializable, Cloneable {
   public void deleteLabel(Label label) {
     //  Ensure selectedItems don't keep a reference to label
     deselectItem(label);
-    label.setLevel(null);
-    // Make a copy of the list to avoid conflicts in the list returned by getLabels
-    this.labels = new ArrayList<Label>(this.labels);
-    this.labels.remove(label);
-    this.labelsChangeSupport.fireCollectionChanged(label, CollectionEvent.Type.DELETE);
+    int index = this.labels.indexOf(label);
+    if (index != -1) {
+      label.setLevel(null);
+      // Make a copy of the list to avoid conflicts in the list returned by getLabels
+      this.labels = new ArrayList<Label>(this.labels);
+      this.labels.remove(index);
+      this.labelsChangeSupport.fireCollectionChanged(label, CollectionEvent.Type.DELETE);
+    }
   }
   
   /**
@@ -949,7 +961,7 @@ public class Home implements Serializable, Cloneable {
    */
   public void setName(String name) {
     if (name != this.name
-        || (name != null && !name.equals(this.name))) {
+        && (name == null || !name.equals(this.name))) {
       String oldName = this.name;
       this.name = name;
       this.propertyChangeSupport.firePropertyChange(Property.NAME.name(), oldName, name);
@@ -1008,7 +1020,7 @@ public class Home implements Serializable, Cloneable {
    */
   public void setFurnitureSortedProperty(HomePieceOfFurniture.SortableProperty furnitureSortedProperty) {
     if (furnitureSortedProperty != this.furnitureSortedProperty
-        || (furnitureSortedProperty != null && !furnitureSortedProperty.equals(this.furnitureSortedProperty))) {
+        && (furnitureSortedProperty == null || !furnitureSortedProperty.equals(this.furnitureSortedProperty))) {
       HomePieceOfFurniture.SortableProperty oldFurnitureSortedProperty = this.furnitureSortedProperty;
       this.furnitureSortedProperty = furnitureSortedProperty;
       this.propertyChangeSupport.firePropertyChange(
@@ -1041,7 +1053,11 @@ public class Home implements Serializable, Cloneable {
    * Returns an unmodifiable list of the furniture properties that are visible.
    */
   public List<HomePieceOfFurniture.SortableProperty> getFurnitureVisibleProperties() {
-    return Collections.unmodifiableList(this.furnitureVisibleProperties);
+    if (this.furnitureVisibleProperties == null) {
+      return Collections.emptyList();
+    } else {
+      return Collections.unmodifiableList(this.furnitureVisibleProperties);
+    }
   }
   
   /**
@@ -1050,7 +1066,7 @@ public class Home implements Serializable, Cloneable {
    */
   public void setFurnitureVisibleProperties(List<HomePieceOfFurniture.SortableProperty> furnitureVisibleProperties) {
     if (furnitureVisibleProperties != this.furnitureVisibleProperties
-        || (furnitureVisibleProperties != null && !furnitureVisibleProperties.equals(this.furnitureVisibleProperties))) {
+        && (furnitureVisibleProperties == null || !furnitureVisibleProperties.equals(this.furnitureVisibleProperties))) {
       List<HomePieceOfFurniture.SortableProperty> oldFurnitureVisibleProperties = this.furnitureVisibleProperties;
       this.furnitureVisibleProperties = new ArrayList<HomePieceOfFurniture.SortableProperty>(furnitureVisibleProperties);
       this.propertyChangeSupport.firePropertyChange(
@@ -1171,8 +1187,7 @@ public class Home implements Serializable, Cloneable {
     if (print != this.print) {
       HomePrint oldPrint = this.print;
       this.print = print;
-      this.propertyChangeSupport.firePropertyChange(
-          Property.PRINT.name(), oldPrint, print);
+      this.propertyChangeSupport.firePropertyChange(Property.PRINT.name(), oldPrint, print);
     }
     this.print = print;
   }
@@ -1257,19 +1272,37 @@ public class Home implements Serializable, Cloneable {
           clone.levels.add(level.clone());
         }
         for (int i = 0; i < this.furniture.size(); i++) {
-          clone.furniture.get(i).setLevel(clone.levels.get(this.levels.indexOf(this.furniture.get(i).getLevel())));
+          Level pieceLevel = this.furniture.get(i).getLevel();
+          if (pieceLevel != null) {
+            // As soon as there's more than one level, every object is supposed to have its level set
+            // but as level can still be null for a undetermined reason, prefer to keep level
+            // to null in the cloned object and having errors further than throwing exception here
+            clone.furniture.get(i).setLevel(clone.levels.get(this.levels.indexOf(pieceLevel)));
+          }
         }
         for (int i = 0; i < this.rooms.size(); i++) {
-          clone.rooms.get(i).setLevel(clone.levels.get(this.levels.indexOf(this.rooms.get(i).getLevel())));
+          Level roomLevel = this.rooms.get(i).getLevel();
+          if (roomLevel != null) {
+            clone.rooms.get(i).setLevel(clone.levels.get(this.levels.indexOf(roomLevel)));
+          }
         }
         for (int i = 0; i < this.dimensionLines.size(); i++) {
-          clone.dimensionLines.get(i).setLevel(clone.levels.get(this.levels.indexOf(this.dimensionLines.get(i).getLevel())));
+          Level dimensionLineLevel = this.dimensionLines.get(i).getLevel();
+          if (dimensionLineLevel != null) {
+            clone.dimensionLines.get(i).setLevel(clone.levels.get(this.levels.indexOf(dimensionLineLevel)));
+          }
         }
         for (int i = 0; i < this.labels.size(); i++) {
-          clone.labels.get(i).setLevel(clone.levels.get(this.levels.indexOf(this.labels.get(i).getLevel())));
+          Level labelLevel = this.labels.get(i).getLevel();
+          if (labelLevel != null) {
+            clone.labels.get(i).setLevel(clone.levels.get(this.levels.indexOf(labelLevel)));
+          }
         }
         for (int i = 0; i < this.walls.size(); i++) {
-          clone.walls.get(i).setLevel(clone.levels.get(this.levels.indexOf(this.walls.get(i).getLevel())));
+          Level wallLevel = this.walls.get(i).getLevel();
+          if (wallLevel != null) {
+            clone.walls.get(i).setLevel(clone.levels.get(this.levels.indexOf(wallLevel)));
+          }
         }
         if (this.selectedLevel != null) {
           clone.selectedLevel = clone.levels.get(this.levels.indexOf(this.selectedLevel));
@@ -1382,14 +1415,6 @@ public class Home implements Serializable, Cloneable {
     return getSubList(items, Label.class);
   }
   
-  /**
-   * Returns a sub list of <code>items</code> that contains only levels.
-   * @since 3.4
-   */
-  public static List<Level> getLevelsSubList(List<? extends Selectable> items) {
-    return getSubList(items, Level.class);
-  }
-  
   /**
    * Returns a sub list of <code>items</code> that contains only instances of <code>subListClass</code>.
    * @since 2.2
