@@ -25,6 +25,7 @@ import java.awt.Color;
 import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.EventQueue;
+import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.GraphicsConfiguration;
 import java.awt.GraphicsEnvironment;
@@ -44,27 +45,27 @@ import java.awt.image.MemoryImageSource;
 import java.io.File;
 import java.io.IOException;
 import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.concurrent.atomic.AtomicReference;
 
 import javax.imageio.ImageIO;
 import javax.media.j3d.AmbientLight;
 import javax.media.j3d.Appearance;
 import javax.media.j3d.Background;
-import javax.media.j3d.BoundingBox;
 import javax.media.j3d.BoundingSphere;
 import javax.media.j3d.BranchGroup;
 import javax.media.j3d.Canvas3D;
 import javax.media.j3d.DirectionalLight;
-import javax.media.j3d.Geometry;
-import javax.media.j3d.GeometryArray;
+import javax.media.j3d.GraphicsConfigTemplate3D;
 import javax.media.j3d.Group;
 import javax.media.j3d.Light;
 import javax.media.j3d.Link;
-import javax.media.j3d.Material;
 import javax.media.j3d.Node;
 import javax.media.j3d.PhysicalBody;
 import javax.media.j3d.PhysicalEnvironment;
-import javax.media.j3d.PolygonAttributes;
 import javax.media.j3d.Shape3D;
+import javax.media.j3d.Texture;
 import javax.media.j3d.Transform3D;
 import javax.media.j3d.TransformGroup;
 import javax.media.j3d.View;
@@ -83,25 +84,33 @@ import javax.vecmath.Vector3d;
 import javax.vecmath.Vector3f;
 
 import com.eteks.sweethome3d.j3d.Component3DManager;
+import com.eteks.sweethome3d.j3d.HomePieceOfFurniture3D;
 import com.eteks.sweethome3d.j3d.ModelManager;
+import com.eteks.sweethome3d.model.CatalogPieceOfFurniture;
 import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.HomeMaterial;
+import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
+import com.sun.j3d.exp.swing.JCanvas3D;
 import com.sun.j3d.utils.universe.SimpleUniverse;
+import com.sun.j3d.utils.universe.Viewer;
 import com.sun.j3d.utils.universe.ViewingPlatform;
 
 /**
  * Super class of 3D preview component for model. 
  */
 public class ModelPreviewComponent extends JComponent {
-  private SimpleUniverse     universe;
-  private JPanel             canvasPanel;
-  private Canvas3D           canvas3D;
-  private BranchGroup        sceneTree;
-  private float              viewYaw = (float) Math.PI / 8;
-  private float              viewPitch = -(float)Math.PI / 16;
-  private float              viewScale = 1;
-  private Object             iconImageLock;
+  private SimpleUniverse          universe;
+  private JPanel                  component3DPanel;
+  private Component               component3D;
+  private BranchGroup             sceneTree;
+  private float                   viewYaw   = (float) Math.PI / 8;
+  private float                   viewPitch = -(float) Math.PI / 16;
+  private float                   viewScale = 1;
+  private Object                  iconImageLock;
+  private HomePieceOfFurniture    previewedPiece;
+  private Map<Texture, Texture>   pieceTextures = new HashMap<Texture, Texture>();
 
   /**
    * Returns an 3D model preview component.
@@ -119,18 +128,18 @@ public class ModelPreviewComponent extends JComponent {
 
     this.sceneTree = createSceneTree();
     
-    this.canvasPanel = new JPanel();
+    this.component3DPanel = new JPanel();
     setLayout(new BorderLayout());
-    add(this.canvasPanel);
+    add(this.component3DPanel);
 
     GraphicsEnvironment graphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
     if (graphicsEnvironment.getScreenDevices().length == 1) {
-      // If only one screen device is available, create canvas 3D immediately, 
+      // If only one screen device is available, create 3D component immediately, 
       // otherwise create it once the screen device of the parent is known
-      createCanvas3D(graphicsEnvironment.getDefaultScreenDevice().getDefaultConfiguration(), pitchAndScaleChangeSupported);
+      createComponent3D(graphicsEnvironment.getDefaultScreenDevice().getDefaultConfiguration(), pitchAndScaleChangeSupported);
     }
 
-    // Add an ancestor listener to create canvas 3D and its universe once this component is made visible 
+    // Add an ancestor listener to create 3D component and its universe once this component is made visible 
     // and clean up universe once its parent frame is disposed
     addAncestorListener(pitchAndScaleChangeSupported);
   }
@@ -148,21 +157,21 @@ public class ModelPreviewComponent extends JComponent {
   }
 
   /**
-   * Returns the component displaying the canvas 3D of this component.
+   * Returns the 3D component of this component.
    */
   JComponent getComponent3D() {
-    return this.canvasPanel;
+    return this.component3DPanel;
   }
   
   /**
-   * Adds an ancestor listener to this component to manage the creation of the canvas and its universe 
+   * Adds an ancestor listener to this component to manage the creation of the 3D component and its universe 
    * and clean up the universe.  
    */
   private void addAncestorListener(final boolean pitchAndScaleChangeSupported) {
     addAncestorListener(new AncestorListener() {
         public void ancestorAdded(AncestorEvent ev) {
-          if (canvas3D == null) {
-            createCanvas3D(ev.getAncestor().getGraphicsConfiguration(), pitchAndScaleChangeSupported);
+          if (component3D == null) {
+            createComponent3D(ev.getAncestor().getGraphicsConfiguration(), pitchAndScaleChangeSupported);
           }
           if (universe == null) {
             createUniverse();
@@ -181,12 +190,28 @@ public class ModelPreviewComponent extends JComponent {
   }
   
   /**
-   * Creates the canvas 3D associated with the given <code>configuration</code> device.
+   * Creates the 3D component associated with the given <code>configuration</code> device.
    */
-  private void createCanvas3D(GraphicsConfiguration graphicsConfiguration, 
+  private void createComponent3D(GraphicsConfiguration graphicsConfiguration, 
                               boolean pitchAndScaleChangeSupported) {
-    this.canvas3D = Component3DManager.getInstance().getOnscreenCanvas3D(graphicsConfiguration,
-        new Component3DManager.RenderingObserver() {
+    if (Boolean.valueOf(System.getProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "false"))) {
+      GraphicsConfigTemplate3D gc = new GraphicsConfigTemplate3D();
+      gc.setSceneAntialiasing(GraphicsConfigTemplate3D.PREFERRED);
+      try {
+        // Instantiate JCanvas3DWithNotifiedPaint inner class by reflection
+        // to be able to run under Java 3D 1.3
+        this.component3D = (Component)Class.forName(ModelPreviewComponent.class.getName() + "$JCanvas3DWithNotifiedPaint").
+            getConstructor(ModelPreviewComponent.class, GraphicsConfigTemplate3D.class).newInstance(this, gc);
+      } catch (ClassNotFoundException ex) {
+        throw new UnsupportedOperationException("Java 3D 1.5 required to display an offscreen 3D view");
+      } catch (Exception ex) {
+        UnsupportedOperationException ex2 = new UnsupportedOperationException();
+        ex2.initCause(ex);
+        throw ex2;
+      }
+    } else {
+      this.component3D = Component3DManager.getInstance().getOnscreenCanvas3D(graphicsConfiguration,
+          new Component3DManager.RenderingObserver() {
             public void canvas3DPreRendered(Canvas3D canvas3d) {
             }
             
@@ -197,19 +222,38 @@ public class ModelPreviewComponent extends JComponent {
               ModelPreviewComponent.this.canvas3DSwapped();
             }            
           });
-    this.canvas3D.setBackground(new Color(0xE5E5E5));
+    }
+    this.component3D.setBackground(new Color(0xE5E5E5));
 
-    // Layout canvas3D
-    this.canvasPanel.setLayout(new GridLayout());
-    this.canvasPanel.add(this.canvas3D);
-    this.canvas3D.setFocusable(false);      
-    addMouseListeners(this.canvas3D, pitchAndScaleChangeSupported);
+    // Layout 3D component
+    this.component3DPanel.setLayout(new GridLayout());
+    this.component3DPanel.add(this.component3D);
+    this.component3D.setFocusable(false);      
+    addMouseListeners(this.component3D, pitchAndScaleChangeSupported);
   }
 
   /**
-   * Adds an AWT mouse listener to canvas that will update view platform transform.  
+   * A <code>JCanvas</code> canvas that sends a notification when it's drawn.
    */
-  private void addMouseListeners(final Canvas3D canvas3D, 
+  private static class JCanvas3DWithNotifiedPaint extends JCanvas3D {
+    private final ModelPreviewComponent homeComponent3D;
+
+    public JCanvas3DWithNotifiedPaint(ModelPreviewComponent component,
+                                      GraphicsConfigTemplate3D template) {
+      super(template);
+      this.homeComponent3D = component;
+    }
+
+    public void paintComponent(Graphics g) {
+      super.paintComponent(g);
+      this.homeComponent3D.canvas3DSwapped();
+    }
+  }
+  
+  /**
+   * Adds an AWT mouse listener to component that will update view platform transform.  
+   */
+  private void addMouseListeners(final Component component3D, 
                                  final boolean pitchAndScaleChangeSupported) {
     final float ANGLE_FACTOR = 0.02f;
     final float ZOOM_FACTOR = 0.02f;
@@ -225,7 +269,7 @@ public class ModelPreviewComponent extends JComponent {
   
         @Override
         public void mouseDragged(MouseEvent ev) {
-          if (getModel() != null) {
+          if (getModelNode() != null) {
             // Mouse move along X axis changes yaw 
             setViewYaw(getViewYaw() - ANGLE_FACTOR * (ev.getX() - this.xLastMouseMove));    
             this.xLastMouseMove = ev.getX();
@@ -244,11 +288,11 @@ public class ModelPreviewComponent extends JComponent {
         }
       };
 
-    canvas3D.addMouseListener(mouseListener);
-    canvas3D.addMouseMotionListener(mouseListener);
+    component3D.addMouseListener(mouseListener);
+    component3D.addMouseMotionListener(mouseListener);
     
     if (pitchAndScaleChangeSupported) {
-      canvas3D.addMouseWheelListener(new MouseWheelListener() {
+      component3D.addMouseWheelListener(new MouseWheelListener() {
           public void mouseWheelMoved(MouseWheelEvent ev) {
             // Mouse move along Y axis with Alt down changes scale
             setViewScale(Math.max(0.5f, Math.min(1.3f, getViewScale() * (float)Math.exp(ev.getWheelRotation() * ZOOM_FACTOR))));
@@ -256,49 +300,64 @@ public class ModelPreviewComponent extends JComponent {
         });
     }
     
-    // Redirect mouse events to the canvas 3D
+    // Redirect mouse events to the 3D component
     for (final MouseListener l : getMouseListeners()) {
-      canvas3D.addMouseListener(new MouseListener() {
+      component3D.addMouseListener(new MouseListener() {
           public void mouseReleased(MouseEvent ev) {
-            l.mouseReleased(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseReleased(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
           
           public void mousePressed(MouseEvent ev) {
-            l.mousePressed(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mousePressed(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
           
           public void mouseExited(MouseEvent ev) {
-            l.mouseExited(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseExited(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
           
           public void mouseEntered(MouseEvent ev) {
-            l.mouseEntered(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseEntered(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
           
           public void mouseClicked(MouseEvent ev) {
-            l.mouseClicked(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseClicked(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
         });
     }
     for (final MouseMotionListener l : getMouseMotionListeners()) {
-      canvas3D.addMouseMotionListener(new MouseMotionListener() {
+      component3D.addMouseMotionListener(new MouseMotionListener() {
           public void mouseMoved(MouseEvent ev) {
-            l.mouseMoved(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseMoved(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
           
           public void mouseDragged(MouseEvent ev) {
-            l.mouseDragged(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, canvas3D));
+            l.mouseDragged(SwingUtilities.convertMouseEvent(ev.getComponent(), ev, component3D));
           }
         });
     }
   }
 
   /**
-   * Creates universe bound to canvas.
+   * Creates universe bound to the 3D component.
    */
   private void createUniverse() {
-    // Link canvas 3D to a default universe
-    this.universe = new SimpleUniverse(this.canvas3D);
+    Canvas3D canvas3D;
+    if (this.component3D instanceof Canvas3D) {
+      canvas3D = (Canvas3D)this.component3D;
+    } else {
+      try {
+        // Call JCanvas3D#getOffscreenCanvas3D by reflection to be able to run under Java 3D 1.3
+        canvas3D = (Canvas3D)Class.forName("com.sun.j3d.exp.swing.JCanvas3D").getMethod("getOffscreenCanvas3D").invoke(this.component3D);
+      } catch (Exception ex) {
+        UnsupportedOperationException ex2 = new UnsupportedOperationException();
+        ex2.initCause(ex);
+        throw ex2;
+      }
+    }    
+    // Create a universe bound to component 3D
+    ViewingPlatform viewingPlatform = new ViewingPlatform();
+    Viewer viewer = new Viewer(canvas3D);
+    this.universe = new SimpleUniverse(viewingPlatform, viewer);
     // Set viewer location 
     updateViewPlatformTransform(this.universe.getViewingPlatform().getViewPlatformTransform(), 
         getViewYaw(), getViewPitch(), getViewScale());
@@ -334,7 +393,7 @@ public class ModelPreviewComponent extends JComponent {
    * Creates a view bound to the universe that views current model from a point of view oriented with 
    * <code>yaw</code> and <code>pitch</code> angles.
    */
-  protected View createView(float yaw, float pitch, float scale, int projectionPolicy) {
+  View createView(float yaw, float pitch, float scale, int projectionPolicy) {
     if (this.universe == null) {
       createUniverse();
     }
@@ -449,9 +508,9 @@ public class ModelPreviewComponent extends JComponent {
     root.setCapability(BranchGroup.ALLOW_DETACH);
     root.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
     // Build scene tree
-    root.addChild(getModelTree());
-    root.addChild(getBackgroundNode());
-    for (Light light : getLights()) {
+    root.addChild(createModelTree());
+    root.addChild(createBackgroundNode());
+    for (Light light : createLights()) {
       root.addChild(light);
     }
     return root;
@@ -460,7 +519,7 @@ public class ModelPreviewComponent extends JComponent {
   /**
    * Returns the background node.  
    */
-  private Node getBackgroundNode() {
+  private Node createBackgroundNode() {
     Background background = new Background(new Color3f(0.9f, 0.9f, 0.9f));
     background.setCapability(Background.ALLOW_COLOR_WRITE);
     background.setApplicationBounds(new BoundingSphere(new Point3d(0, 0, 0), 100));
@@ -478,11 +537,12 @@ public class ModelPreviewComponent extends JComponent {
   /**
    * Returns the lights of the scene.
    */
-  private Light [] getLights() {
+  private Light [] createLights() {
     Light [] lights = {
         new DirectionalLight(new Color3f(0.9f, 0.9f, 0.9f), new Vector3f(1.732f, -0.8f, -1)), 
         new DirectionalLight(new Color3f(0.9f, 0.9f, 0.9f), new Vector3f(-1.732f, -0.8f, -1)), 
         new DirectionalLight(new Color3f(0.9f, 0.9f, 0.9f), new Vector3f(0, -0.8f, 1)), 
+        new DirectionalLight(new Color3f(0.66f, 0.66f, 0.66f), new Vector3f(0, 1f, 0)), 
         new AmbientLight(new Color3f(0.2f, 0.2f, 0.2f))}; 
 
     for (Light light : lights) {
@@ -494,7 +554,7 @@ public class ModelPreviewComponent extends JComponent {
   /**
    * Returns the root of model tree.
    */
-  private Node getModelTree() {
+  private Node createModelTree() {
     TransformGroup modelTransformGroup = new TransformGroup();      
     //  Allow transform group to have new children
     modelTransformGroup.setCapability(Group.ALLOW_CHILDREN_READ);
@@ -506,165 +566,101 @@ public class ModelPreviewComponent extends JComponent {
   }
 
   /**
-   * Returns the <code>model</code> displayed by this component. 
+   * Returns the 3D model content displayed by this component.
    */
-  public BranchGroup getModel() {
+  public Content getModel() {
+    return this.previewedPiece.getModel();
+  }
+
+  /**
+   * Sets the 3D model content displayed by this component. 
+   * The model is shown at its default orientation and in a box 1 unit wide.
+   */
+  public void setModel(final Content model) {
+    final TransformGroup modelTransformGroup = (TransformGroup)this.sceneTree.getChild(0);
+    modelTransformGroup.removeAllChildren();
+    this.previewedPiece = null;
+    this.pieceTextures.clear();
+    if (model != null) {
+      final AtomicReference<IllegalArgumentException> exception = new AtomicReference<IllegalArgumentException>();
+      // Load content model synchronously (or get it from cache)
+      ModelManager.getInstance().loadModel(model, true, new ModelManager.ModelObserver() {        
+          public void modelUpdated(BranchGroup modelRoot) {
+            if (modelRoot.numChildren() > 0) {
+              try {
+                Vector3f size = ModelManager.getInstance().getSize(modelRoot);
+                previewedPiece = new HomePieceOfFurniture(
+                    new CatalogPieceOfFurniture(null, null, model, 
+                        size.x, size.z, size.y, 0, false, null, null, false, 0, false));
+                previewedPiece.setX(0);
+                previewedPiece.setY(0);
+                previewedPiece.setElevation(-previewedPiece.getHeight() / 2);
+                
+                Transform3D modelTransform = new Transform3D();
+                modelTransform.setScale(1.8 / Math.max(Math.max(size.x, size.z), size.y));
+                modelTransformGroup.setTransform(modelTransform);
+
+                modelTransformGroup.addChild(new HomePieceOfFurniture3D(previewedPiece, null, true, true));
+              } catch (IllegalArgumentException ex) {
+                // Model is empty
+              }
+            }
+          }
+          
+          public void modelError(Exception ex) {
+            exception.set(new IllegalArgumentException("Couldn't load model", ex));
+          }
+        });
+      
+      if (exception.get() != null) {
+        throw exception.get(); 
+      }
+    }
+  }
+
+  /**
+   * Sets the back face visibility of the children nodes of the displayed 3D model.
+   */
+  protected void setBackFaceShown(boolean backFaceShown) {
+    if (this.previewedPiece != null) {
+      // Create a new piece from the existing one with an updated backFaceShown flag 
+      this.previewedPiece = new HomePieceOfFurniture(
+          new CatalogPieceOfFurniture(null, null, this.previewedPiece.getModel(), 
+              this.previewedPiece.getWidth(), 
+              this.previewedPiece.getDepth(),
+              this.previewedPiece.getHeight(),
+              0, false, this.previewedPiece.getColor(), null, backFaceShown, 0, false));
+      this.previewedPiece.setX(0);
+      this.previewedPiece.setY(0);
+      this.previewedPiece.setElevation(-previewedPiece.getHeight() / 2);
+    
+      TransformGroup modelTransformGroup = (TransformGroup)this.sceneTree.getChild(0);
+      modelTransformGroup.addChild(new HomePieceOfFurniture3D(this.previewedPiece, null, true, true));
+      if (modelTransformGroup.numChildren() > 1) {
+        modelTransformGroup.removeChild(0);
+      }
+    }
+  }
+
+  /**
+   * Returns the 3D model node displayed by this component. 
+   */
+  private HomePieceOfFurniture3D getModelNode() {
     TransformGroup modelTransformGroup = (TransformGroup)this.sceneTree.getChild(0);
     if (modelTransformGroup.numChildren() > 0) {
-      return (BranchGroup)modelTransformGroup.getChild(0);
+      return (HomePieceOfFurniture3D)modelTransformGroup.getChild(0);
     } else {
       return null;
     }
   }
   
   /**
-   * Sets the <code>model</code> displayed by this component. 
-   * The model is shown at its default orientation and in a box 1 unit wide.
-   */
-  public void setModel(BranchGroup model) {
-    TransformGroup modelTransformGroup = (TransformGroup)this.sceneTree.getChild(0);
-    modelTransformGroup.removeAllChildren();
-    if (model != null) {
-      model = (BranchGroup)ModelManager.getInstance().cloneNode(model);
-      model.setCapability(BranchGroup.ALLOW_DETACH);
-      setNodeCapabilities(model);
-      
-      if (model.numChildren() > 0) {
-        BoundingBox bounds = null;
-        try {
-          bounds = ModelManager.getInstance().getBounds(model);
-        } catch (IllegalArgumentException ex) {
-          // Model is empty
-        }
-        if (bounds != null) {
-          Point3d lower = new Point3d();
-          bounds.getLower(lower);
-          Point3d upper = new Point3d();
-          bounds.getUpper(upper);
-          
-          // Translate model to center
-          Transform3D translation = new Transform3D ();
-          translation.setTranslation(
-              new Vector3d(-lower.x - (upper.x - lower.x) / 2, 
-                           -lower.y - (upper.y - lower.y) / 2, 
-                           -lower.z - (upper.z - lower.z) / 2));
-          // Scale model to make it fit in a 1.8 unit wide box
-          Transform3D modelTransform = new Transform3D();
-          modelTransform.setScale (1.8 / Math.max (Math.max (upper.x -lower.x, upper.y - lower.y), 
-                                                   upper.z - lower.z));
-          modelTransform.mul(translation);
-          
-          modelTransformGroup.setTransform(modelTransform);
-          modelTransformGroup.addChild(model);
-        }
-      }
-    }
-  }
-  
-  /**
-   * Sets the capability to read bounds, to write polygon and material attributes  
-   * for all children of <code>node</code>.
-   */
-  private void setNodeCapabilities(Node node) {
-    if (node instanceof Group) {
-      node.setCapability(Group.ALLOW_CHILDREN_READ);
-      if (node instanceof TransformGroup) {
-        node.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
-      }
-      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
-      while (enumeration.hasMoreElements()) {
-        setNodeCapabilities((Node)enumeration.nextElement());
-      }
-    } else if (node instanceof Link) {
-      node.setCapability(Link.ALLOW_SHARED_GROUP_READ);
-      setNodeCapabilities(((Link)node).getSharedGroup());
-    } else if (node instanceof Shape3D) {
-      Shape3D shape = (Shape3D)node;
-      node.setCapability(Node.ALLOW_BOUNDS_READ);
-      Appearance appearance = ((Shape3D)node).getAppearance();
-      if (appearance == null) {
-        appearance = new Appearance();
-        shape.setAppearance(appearance);
-      }
-      appearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
-      appearance.setCapability(Appearance.ALLOW_MATERIAL_READ);
-      appearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
-      node.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
-
-      PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
-      if (polygonAttributes == null) {
-        polygonAttributes = new PolygonAttributes();
-        appearance.setPolygonAttributes(polygonAttributes);
-      }
-      polygonAttributes.setCapability(PolygonAttributes.ALLOW_CULL_FACE_WRITE);
-      polygonAttributes.setCapability(PolygonAttributes.ALLOW_NORMAL_FLIP_WRITE);
-      
-      Enumeration<?> enumeration = shape.getAllGeometries();
-      while (enumeration.hasMoreElements()) {
-        Geometry geometry = (Geometry) enumeration.nextElement();
-        if (!geometry.isLive()
-            && geometry instanceof GeometryArray) {
-          geometry.setCapability(GeometryArray.ALLOW_FORMAT_READ);
-          geometry.setCapability(GeometryArray.ALLOW_COUNT_READ);
-          geometry.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
-          geometry.setCapability(GeometryArray.ALLOW_NORMAL_READ);
-          geometry.setCapability(GeometryArray.ALLOW_TEXCOORD_READ);
-          geometry.setCapability(GeometryArray.ALLOW_REF_DATA_READ);
-        }
-      }
-    }
-  }
-  
-  /**
-   * Sets the back face visibility of all <code>Shape3D</code> children nodes of displayed model.
-   */
-  protected void setBackFaceShown(boolean backFaceShown) {
-    setBackFaceShown(this.sceneTree.getChild(0), backFaceShown);
-  }
-  
-  /**
-   * Sets the back face visibility of all <code>Shape3D</code> children nodes of <code>node</code>.
-   */
-  private void setBackFaceShown(Node node, boolean backFaceShown) {
-    if (node instanceof Group) {
-      // Set visibility of all children
-      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
-      while (enumeration.hasMoreElements()) {
-        setBackFaceShown((Node)enumeration.nextElement(), backFaceShown);
-      }
-    } else if (node instanceof Link) {
-      setBackFaceShown(((Link)node).getSharedGroup(), backFaceShown);
-    } else if (node instanceof Shape3D) {
-      Appearance appearance = ((Shape3D)node).getAppearance();
-      PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
-      // Change cull face
-      polygonAttributes.setCullFace(backFaceShown 
-          ? PolygonAttributes.CULL_FRONT
-          : PolygonAttributes.CULL_BACK);
-      // Change back face normal flip
-      polygonAttributes.setBackFaceNormalFlip(backFaceShown);
-    }
-  }
-
-  /**
-   * Updates the rotation of the model displayed by this component. 
+   * Updates the rotation of the 3D model displayed by this component. 
    * The model is shown at its default size.
    */
   protected void setModelRotation(float [][] modelRotation) {
-    BranchGroup model = getModel();
-    if (model != null && model.numChildren() > 0) {
-      BoundingBox bounds = ModelManager.getInstance().getBounds(model);
-      Point3d lower = new Point3d();
-      bounds.getLower(lower);
-      Point3d upper = new Point3d();
-      bounds.getUpper(upper);
-    
-      // Translate model to center
-      Transform3D translation = new Transform3D ();
-      translation.setTranslation(
-          new Vector3d(-lower.x - (upper.x - lower.x) / 2, 
-                       -lower.y - (upper.y - lower.y) / 2, 
-                       -lower.z - (upper.z - lower.z) / 2));
+    BranchGroup modelNode = getModelNode();
+    if (modelNode != null && modelNode.numChildren() > 0) {
       // Apply model rotation
       Transform3D rotationTransform = new Transform3D();
       if (modelRotation != null) {
@@ -673,10 +669,10 @@ public class ModelPreviewComponent extends JComponent {
             modelRotation [2][0], modelRotation [2][1], modelRotation [2][2]);
         rotationTransform.setRotation(modelRotationMatrix);
       }
-      rotationTransform.mul(translation);
       // Scale model to make it fit in a 1.8 unit wide box      
       Transform3D modelTransform = new Transform3D();
-      modelTransform.setScale(1.8 / Math.max(Math.max((upper.x -lower.x), (upper.z - lower.z)), (upper.y - lower.y)));
+      Vector3f size = ModelManager.getInstance().getSize(modelNode);
+      modelTransform.setScale(1.8 / Math.max(Math.max(size.x, size.z), size.y));
       modelTransform.mul(rotationTransform);
       
       TransformGroup modelTransformGroup = (TransformGroup)this.sceneTree.getChild(0);
@@ -685,13 +681,13 @@ public class ModelPreviewComponent extends JComponent {
   }
   
   /**
-   * Updates the rotation and the size of the model displayed by this component. 
+   * Updates the rotation and the size of the 3D model displayed by this component. 
    */
   protected void setModelRotationAndSize(float [][] modelRotation,
                                          float width, float depth, float height) {
-    BranchGroup model = getModel();
-    if (model != null && model.numChildren() > 0) {
-      Transform3D normalization = ModelManager.getInstance().getNormalizedTransform(model, modelRotation, 1f);
+    BranchGroup modelNode = getModelNode();
+    if (modelNode != null && modelNode.numChildren() > 0) {
+      Transform3D normalization = ModelManager.getInstance().getNormalizedTransform(modelNode, modelRotation, 1f);
       // Scale model to its size
       Transform3D scaleTransform = new Transform3D();
       if (width != 0 && depth != 0 && height != 0) {
@@ -703,7 +699,7 @@ public class ModelPreviewComponent extends JComponent {
       if (width != 0 && depth != 0 && height != 0) {
         modelTransform.setScale(1.8 / Math.max(Math.max(width, height), depth));
       } else {
-        Vector3f size = ModelManager.getInstance().getSize(model);
+        Vector3f size = ModelManager.getInstance().getSize(modelNode);
         modelTransform.setScale(1.8 / Math.max(Math.max(size.x, size.z), size.y));
       }
       modelTransform.mul(scaleTransform);
@@ -714,56 +710,54 @@ public class ModelPreviewComponent extends JComponent {
   }
 
   /**
-   * Sets the color applied to piece model.
+   * Sets the color applied to 3D model.
    */
   protected void setModelColor(Integer color) {
-    if (color != null) {
-      Color3f materialColor = new Color3f(((color >>> 16) & 0xFF) / 255f,
-                                           ((color >>> 8) & 0xFF) / 255f,
-                                                   (color & 0xFF) / 255f);
-      setMaterial(this.sceneTree.getChild(0), 
-          new Material(materialColor, new Color3f(), materialColor, materialColor, 64));
-    } else {
-      // Set default material of model
-      setMaterial(this.sceneTree.getChild(0), null);
+    if (this.previewedPiece != null
+        && this.previewedPiece.getColor() != color) {
+      this.previewedPiece.setColor(color);
+      getModelNode().update();
     }
   }
 
   /**
-   * Sets the material attribute of all <code>Shape3D</code> children nodes of <code>node</code> 
-   * with a given <code>material</code>. 
+   * Sets the materials applied to 3D model.
    */
-  private void setMaterial(Node node, Material material) {
+  public void setModelMaterials(HomeMaterial [] materials) {
+    if (this.previewedPiece != null) {
+      this.previewedPiece.setModelMaterials(materials);
+      getModelNode().update();
+      // Replace textures by clones because Java 3D doesn't accept all the time to share textures 
+      cloneTexture(getModelNode(), this.pieceTextures);
+    }
+  }
+
+  /**
+   * Replace the textures set on <code>node</code> shapes by clones. 
+   */
+  private void cloneTexture(Node node, Map<Texture, Texture> replacedTextures) {
     if (node instanceof Group) {
-      // Set material of all children
+      // Enumerate children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        setMaterial((Node)enumeration.nextElement(), material);
+        cloneTexture((Node)enumeration.nextElement(), replacedTextures);
       }
     } else if (node instanceof Link) {
-      setMaterial(((Link)node).getSharedGroup(), material);
+      cloneTexture(((Link)node).getSharedGroup(), replacedTextures);
     } else if (node instanceof Shape3D) {
-      Shape3D shape = (Shape3D)node;
-      String shapeName = (String)shape.getUserData();
-      // Change material of all shape that are not window panes
-      if (shapeName == null
-          || !shapeName.startsWith(ModelManager.WINDOW_PANE_SHAPE_PREFIX)) {
-        Appearance appearance = shape.getAppearance();
-        // Use appearance user data to store shape default material
-        Material defaultMaterial = (Material)appearance.getUserData();
-        if (defaultMaterial == null) {
-          defaultMaterial = appearance.getMaterial();
-          appearance.setUserData(defaultMaterial);
-        }
-        // Change material
-        if (material != null) {
-          appearance.setMaterial(material);
-        } else {
-          // Restore default material
-          appearance.setMaterial(defaultMaterial);
+      Appearance appearance = ((Shape3D)node).getAppearance();
+      if (appearance != null) {
+        Texture texture = appearance.getTexture();
+        if (texture != null) {
+          Texture replacedTexture = replacedTextures.get(texture);
+          if (replacedTexture == null) {
+            replacedTexture = (Texture)texture.cloneNodeComponent(false);
+            replacedTextures.put(texture, replacedTexture);
+          }
+          appearance.setTexture(replacedTexture);
         }
       }
-    }
+    } 
   }
 
   /**
@@ -778,9 +772,9 @@ public class ModelPreviewComponent extends JComponent {
     this.iconImageLock = new Object();
     try {
       // Instead of using off screen images that may cause some problems
-      // use Robot to capture canvas 3D image instead of
-      Point canvas3DOrigin = new Point();
-      SwingUtilities.convertPointToScreen(canvas3DOrigin, this.canvas3D);
+      // use Robot to capture canvas 3D image 
+      Point component3DOrigin = new Point();
+      SwingUtilities.convertPointToScreen(component3DOrigin, this.component3D);
       
       Robot robot = new Robot();
       // Render scene with a white background
@@ -795,7 +789,7 @@ public class ModelPreviewComponent extends JComponent {
         }
       }
       imageWithWhiteBackgound = robot.createScreenCapture(
-          new Rectangle(canvas3DOrigin, this.canvas3D.getSize()));
+          new Rectangle(component3DOrigin, this.component3D.getSize()));
       
       // Render scene with a black background
       if (this.iconImageLock != null) {
@@ -809,7 +803,7 @@ public class ModelPreviewComponent extends JComponent {
         }
       }
       imageWithBlackBackgound = robot.createScreenCapture(
-          new Rectangle(canvas3DOrigin, this.canvas3D.getSize()));
+          new Rectangle(component3DOrigin, this.component3D.getSize()));
     } catch (AWTException ex) {
       throw new RuntimeException(ex);
     } finally {
@@ -845,7 +839,7 @@ public class ModelPreviewComponent extends JComponent {
   }
 
   /**
-   * Returns the icon content matching the displayed view.
+   * Returns a temporary content of the icon matching the displayed view.
    */
   public Content getIcon(int maxWaitingDelay) throws IOException {
     File tempIconFile = OperatingSystem.createTemporaryFile("icon", ".png");
